// Copyright © 2020 Amazon Web Services
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package org.opengroup.osdu.notification.provider.aws.impl;


import com.amazonaws.auth.AWSCredentialsProvider;
import com.amazonaws.services.simplesystemsmanagement.AWSSimpleSystemsManagement;
import com.amazonaws.services.simplesystemsmanagement.AWSSimpleSystemsManagementClientBuilder;
import com.amazonaws.services.simplesystemsmanagement.model.GetParameterRequest;
import com.amazonaws.services.simplesystemsmanagement.model.GetParameterResult;
import com.amazonaws.services.simplesystemsmanagement.model.Parameter;
import lombok.AccessLevel;
import lombok.Getter;
import lombok.Setter;

import org.opengroup.osdu.core.aws.entitlements.ServicePrincipal;
import org.opengroup.osdu.core.aws.iam.IAMConfig;
import org.opengroup.osdu.core.aws.secrets.SecretsManager;
import org.opengroup.osdu.core.common.util.IServiceAccountJwtClient;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;


@Component
public class ServiceAccountJwtAwsClientImpl implements IServiceAccountJwtClient {


    @Value("${aws.region}")
    @Getter()
    @Setter(AccessLevel.PROTECTED)
    public String amazonRegion;



    @Value("${aws.ssm}")
    @Getter()
    @Setter(AccessLevel.PROTECTED)
    public Boolean ssmEnabled;


    @Value("${aws.environment}")
    @Getter()
    @Setter(AccessLevel.PROTECTED)
    public String environment;


    @Value("${aws.tokenUrl}")
    @Getter()
    public String tokenUrl;

    @Value("${aws.oauth.custom.scope}")
    @Getter()
    private String awsOauthCustomScope;



    String client_credentials_secret;
    String client_credentials_clientid;
    ServicePrincipal sp;


    private AWSCredentialsProvider amazonAWSCredentials;
    private AWSSimpleSystemsManagement ssmManager;

    @PostConstruct
    public void init() {
        if (ssmEnabled) {

            SecretsManager sm = new SecretsManager();
            sp = new ServicePrincipal(amazonRegion,environment,tokenUrl,awsOauthCustomScope);

            String client_credentials_client_id = "/osdu/" + environment + "/client-credentials-client-id";
            String client_secret_key = "client_credentials_client_secret";
            String client_secret_secretName = "/osdu/" + environment + "/client_credentials_secret";

            amazonAWSCredentials = IAMConfig.amazonAWSCredentials();
            ssmManager = AWSSimpleSystemsManagementClientBuilder.standard()
                    .withCredentials(amazonAWSCredentials)
                    .withRegion(amazonRegion)
                    .build();

            GetParameterRequest paramRequest = new GetParameterRequest()
                    .withName(client_credentials_client_id)
                    .withWithDecryption(true);
            GetParameterResult paramResult = ssmManager.getParameter(paramRequest);
            Parameter paramsResult = paramResult.getParameter();
            client_credentials_clientid = paramsResult.getValue();
            client_credentials_secret = sm.getSecret(client_secret_secretName,amazonRegion,client_secret_key);

        }
    }

    @Override
    public String getIdToken(String s) {
        String token=  sp.getServicePrincipalAccessToken(client_credentials_clientid,client_credentials_secret);
        System.out.println("This is the client credentials token = " + token);
        return token;
    }


}
