/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.notification.util;

public class Config {
    public String HostUrl;
    public String OsduTenant;
    public String ClientTenant;
    public String IntegrationAudience;
    public String GSAPushUrl;
    public String HMACPushUrl;
    public String RegisterServicePath;
    public String Topic;
    public String hmacSecretValue;
    public String NotificationId;
    public String DE_OPS_TESTER = System.getProperty("DE_OPS_TESTER", System.getenv("DE_OPS_TESTER"));

    private static Config config = new Config();

    public static Config Instance() {
        String env = getEnvironment();
        config.ClientTenant = getEnvironmentVariableOrDefaultValue("CLIENT_TENANT","nonexistenttenant");
        config.IntegrationAudience =  getEnvironmentVariableOrDefaultValue("INTEGRATION_TEST_AUDIENCE","245464679631-ktfdfpl147m1mjpbutl00b3cmffissgq.apps.googleusercontent.com");
        config.OsduTenant = getEnvironmentVariableOrDefaultValue("OSDU_TENANT","opendes");
        config.Topic = getEnvironmentVariableOrDefaultValue("TOPIC_ID","records-changed");

        config.hmacSecretValue = System.getProperty("HMAC_SECRET", System.getenv("HMAC_SECRET"));
        if (env.equalsIgnoreCase("LOCAL")) {
            //make sure to run register service on a different port. You can also choose to point to Register service that is running in cloud
            String registerUrl = "http://localhost:8081/api/register/v1";

            //must have notification and register services running on different ports
            config.HostUrl = "http://localhost:8080/";

            config.GSAPushUrl = registerUrl+"/test/gsa-challenge/";
            config.HMACPushUrl = registerUrl+"/test/challenge/";
            config.RegisterServicePath = registerUrl;
        } else if (env.equalsIgnoreCase("DEV") || isGke()) {
            String registerUrl = System.getProperty("REGISTER_BASE_URL", System.getenv("REGISTER_BASE_URL"));
            config.HostUrl = System.getProperty("NOTIFICATION_BASE_URL", System.getenv("NOTIFICATION_BASE_URL"));
            config.GSAPushUrl = registerUrl+"/test/gsa-challenge/";
            //Adding this so CPs can point to custom HMAC push endpoints
            config.HMACPushUrl = getEnvironmentVariableOrDefaultValue("REGISTER_CUSTOM_PUSH_URL_HMAC",registerUrl+"/test/challenge/");
            //Adding a new variable NOTIFICATION_REGISTER_BASE_URL since REGISTER_BASE_URL is used by Register integration tests which needs a trailing \
             String regUrl= getEnvironmentVariable("NOTIFICATION_REGISTER_BASE_URL");
             if(regUrl==null)
             {
                 config.RegisterServicePath = registerUrl;
             }
             else
             {
                 config.RegisterServicePath = regUrl+"/api/register/v1";
             }

        }else
            throw new RuntimeException("$ENVIRONMENT environment variable not provided");

        System.out.println("HostUrl="+config.HostUrl);
        System.out.println("config.Topic="+ config.Topic);
        System.out.println("config.HMACPushUrl="+ config.HMACPushUrl);
        System.out.println("config.RegisterServicePath="+ config.RegisterServicePath);
        return config;
    }

    public boolean isLocalHost() {
        return HostUrl.contains("//localhost");
    }

    public static boolean isGke() {
        return "DEV_GKE".equalsIgnoreCase(getEnvironment());
    }

    private static String getEnvironment() {
        return System.getProperty("ENVIRONMENT", System.getenv("ENVIRONMENT"));
    }

    private static String getEnvironmentVariableOrDefaultValue(String key, String defaultValue) {
        String environmentVariable = getEnvironmentVariable(key);
        if (environmentVariable == null) {
            environmentVariable = defaultValue;
        }
        return environmentVariable;
    }

    private static String getEnvironmentVariable(String propertyKey) {
        return System.getProperty(propertyKey, System.getenv(propertyKey));
    }
}
